import os
try:
    from PySide import QtCore, QtGui
except ImportError:
    from PyQt4 import QtCore, QtGui


MAX_SHORT_NAME_LENGTH = 5

LOSSY_FIRST_BYTE = 200

LOSSLESS_FIRST_BYTE = 160


def path_to_data(name):
    """
    :param name: plugin unique name
    :return path do plugin's directory
    """
    return os.path.dirname(os.path.realpath(__file__)) + '/' + name + '/'


def log(name, data):
    """
    :param name: plugin unique name
    :param data: data for saving in log
    """
    with open(path_to_data(name) + 'logs.txt', 'a') as fl:
        fl.write(bytes(data, 'utf-8') + b'\n')


class PluginSuperClass:
    """
    Superclass for all plugins. Plugin is python module with at least one class derived from PluginSuperClass.
    """
    is_plugin = True

    def __init__(self, name, short_name, tox=None, profile=None, settings=None, encrypt_save=None):
        """
        Constructor. In plugin __init__ should take only 4 last arguments
        :param name: plugin full name
        :param short_name: plugin unique short name (length of short name should not exceed MAX_SHORT_NAME_LENGTH)
        :param tox: tox instance
        :param profile: profile instance
        :param settings: profile settings
        :param encrypt_save: LibToxEncryptSave instance.
        """
        self._settings = settings
        self._profile = profile
        self._tox = tox
        name = name.strip()
        short_name = short_name.strip()
        if not name or not short_name:
            raise NameError('Wrong name')
        self._name = name
        self._short_name = short_name[:MAX_SHORT_NAME_LENGTH]
        self._translator = None  # translator for plugin's GUI
        self._encrypt_save = encrypt_save

    # -----------------------------------------------------------------------------------------------------------------
    # Get methods
    # -----------------------------------------------------------------------------------------------------------------

    def get_name(self):
        """
        :return plugin full name
        """
        return self._name

    def get_short_name(self):
        """
        :return plugin unique (short) name
        """
        return self._short_name

    def get_description(self):
        """
        Should return plugin description
        """
        return self.__doc__

    def get_menu(self, menu, row_number):
        """
        This method creates items for menu which called on right click in list of friends
        :param menu: menu instance
        :param row_number: number of selected row in list of contacts
        :return list of QAction's
        """
        return []

    def get_window(self):
        """
        This method should return window for plugins with GUI or None
        """
        return None

    def set_tox(self, tox):
        """
        New tox instance
        """
        self._tox = tox

    # -----------------------------------------------------------------------------------------------------------------
    # Plugin was stopped, started or new command received
    # -----------------------------------------------------------------------------------------------------------------

    def start(self):
        """
        This method called when plugin was started
        """
        pass

    def stop(self):
        """
        This method called when plugin was stopped
        """
        pass

    def close(self):
        """
        App is closing
        """
        pass

    def command(self, command):
        """
        New command. On 'help' this method should provide user list of available commands
        :param command: string with command
        """
        if command == 'help':
            msgbox = QtGui.QMessageBox()
            title = QtGui.QApplication.translate("PluginWindow", "List of commands for plugin {}", None, QtGui.QApplication.UnicodeUTF8)
            msgbox.setWindowTitle(title.format(self._name))
            msgbox.setText(QtGui.QApplication.translate("PluginWindow", "No commands available", None, QtGui.QApplication.UnicodeUTF8))
            msgbox.exec_()

    # -----------------------------------------------------------------------------------------------------------------
    # Translations support
    # -----------------------------------------------------------------------------------------------------------------

    def load_translator(self):
        """
        This method loads translations for GUI
        """
        app = QtGui.QApplication.instance()
        langs = self._settings.supported_languages()
        curr_lang = self._settings['language']
        if curr_lang in langs:
            if self._translator is not None:
                app.removeTranslator(self._translator)
            self._translator = QtCore.QTranslator()
            lang_path = langs[curr_lang]
            self._translator.load(path_to_data(self._short_name) + lang_path)
            app.installTranslator(self._translator)

    # -----------------------------------------------------------------------------------------------------------------
    # Settings loading and saving
    # -----------------------------------------------------------------------------------------------------------------

    def load_settings(self):
        """
        This method loads settings of plugin and returns raw data
        """
        with open(path_to_data(self._short_name) + 'settings.json', 'rb') as fl:
            data = fl.read()
        return str(data, 'utf-8')

    def save_settings(self, data):
        """
        This method saves plugin's settings to file
        :param data: string with data
        """
        with open(path_to_data(self._short_name) + 'settings.json', 'wb') as fl:
            fl.write(bytes(data, 'utf-8'))

    # -----------------------------------------------------------------------------------------------------------------
    # Callbacks
    # -----------------------------------------------------------------------------------------------------------------

    def lossless_packet(self, data, friend_number):
        """
        Incoming lossless packet
        :param data: string with data
        :param friend_number: number of friend who sent packet
        """
        pass

    def lossy_packet(self, data, friend_number):
        """
        Incoming lossy packet
        :param data: string with data
        :param friend_number: number of friend who sent packet
        """
        pass

    def friend_connected(self, friend_number):
        """
        Friend with specified number is online now
        """
        pass

    # -----------------------------------------------------------------------------------------------------------------
    # Custom packets sending
    # -----------------------------------------------------------------------------------------------------------------

    def send_lossless(self, data, friend_number):
        """
        This method sends lossless packet to friend
        Wrapper for self._tox.friend_send_lossless_packet
        Use it instead of direct using self._tox.friend_send_lossless_packet
        :return True on success
        """
        if data is None:
            data = ''
        try:
            return self._tox.friend_send_lossless_packet(friend_number,
                                                         bytes([ord(x) for x in
                                                                chr(len(self._short_name) + LOSSLESS_FIRST_BYTE) +
                                                                self._short_name + str(data)
                                                                ]))
        except:
            return False

    def send_lossy(self, data, friend_number):
        """
        This method sends lossy packet to friend
        Wrapper for self._tox.friend_send_lossy_packet
        Use it instead of direct using self._tox.friend_send_lossy_packet
        :return True on success
        """
        if data is None:
            data = ''
        try:
            return self._tox.friend_send_lossy_packet(friend_number,
                                                      bytes([ord(x) for x in
                                                             chr(len(self._short_name) + LOSSY_FIRST_BYTE) +
                                                             self._short_name + str(data)
                                                             ]))
        except:
            return False



TOX_USER_STATUS = {
    'NONE': 0,
    'AWAY': 1,
    'BUSY': 2,
}

TOX_MESSAGE_TYPE = {
    'NORMAL': 0,
    'ACTION': 1,
}

TOX_PROXY_TYPE = {
    'NONE': 0,
    'HTTP': 1,
    'SOCKS5': 2,
}

TOX_SAVEDATA_TYPE = {
    'NONE': 0,
    'TOX_SAVE': 1,
    'SECRET_KEY': 2,
}

TOX_ERR_OPTIONS_NEW = {
    'OK': 0,
    'MALLOC': 1,
}

TOX_ERR_NEW = {
    'OK': 0,
    'NULL': 1,
    'MALLOC': 2,
    'PORT_ALLOC': 3,
    'PROXY_BAD_TYPE': 4,
    'PROXY_BAD_HOST': 5,
    'PROXY_BAD_PORT': 6,
    'PROXY_NOT_FOUND': 7,
    'LOAD_ENCRYPTED': 8,
    'LOAD_BAD_FORMAT': 9,
}

TOX_ERR_BOOTSTRAP = {
    'OK': 0,
    'NULL': 1,
    'BAD_HOST': 2,
    'BAD_PORT': 3,
}

TOX_CONNECTION = {
    'NONE': 0,
    'TCP': 1,
    'UDP': 2,
}

TOX_ERR_SET_INFO = {
    'OK': 0,
    'NULL': 1,
    'TOO_LONG': 2,
}

TOX_ERR_FRIEND_ADD = {
    'OK': 0,
    'NULL': 1,
    'TOO_LONG': 2,
    'NO_MESSAGE': 3,
    'OWN_KEY': 4,
    'ALREADY_SENT': 5,
    'BAD_CHECKSUM': 6,
    'SET_NEW_NOSPAM': 7,
    'MALLOC': 8,
}

TOX_ERR_FRIEND_DELETE = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
}

TOX_ERR_FRIEND_BY_PUBLIC_KEY = {
    'OK': 0,
    'NULL': 1,
    'NOT_FOUND': 2,
}

TOX_ERR_FRIEND_GET_PUBLIC_KEY = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
}

TOX_ERR_FRIEND_GET_LAST_ONLINE = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
}

TOX_ERR_FRIEND_QUERY = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
}

TOX_ERR_SET_TYPING = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
}

TOX_ERR_FRIEND_SEND_MESSAGE = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
    'FRIEND_NOT_CONNECTED': 3,
    'SENDQ': 4,
    'TOO_LONG': 5,
    'EMPTY': 6,
}

TOX_FILE_KIND = {
    'DATA': 0,
    'AVATAR': 1,
}

TOX_FILE_CONTROL = {
    'RESUME': 0,
    'PAUSE': 1,
    'CANCEL': 2,
}

TOX_ERR_FILE_CONTROL = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
    'FRIEND_NOT_CONNECTED': 2,
    'NOT_FOUND': 3,
    'NOT_PAUSED': 4,
    'DENIED': 5,
    'ALREADY_PAUSED': 6,
    'SENDQ': 7,
}

TOX_ERR_FILE_SEEK = {
    'OK': 0,
    'FRIEND_NOT_FOUND': 1,
    'FRIEND_NOT_CONNECTED': 2,
    'NOT_FOUND': 3,
    'DENIED': 4,
    'INVALID_POSITION': 5,
    'SENDQ': 6,
}

TOX_ERR_FILE_GET = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
    'NOT_FOUND': 3,
}

TOX_ERR_FILE_SEND = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
    'FRIEND_NOT_CONNECTED': 3,
    'NAME_TOO_LONG': 4,
    'TOO_MANY': 5,
}

TOX_ERR_FILE_SEND_CHUNK = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
    'FRIEND_NOT_CONNECTED': 3,
    'NOT_FOUND': 4,
    'NOT_TRANSFERRING': 5,
    'INVALID_LENGTH': 6,
    'SENDQ': 7,
    'WRONG_POSITION': 8,
}

TOX_ERR_FRIEND_CUSTOM_PACKET = {
    'OK': 0,
    'NULL': 1,
    'FRIEND_NOT_FOUND': 2,
    'FRIEND_NOT_CONNECTED': 3,
    'INVALID': 4,
    'EMPTY': 5,
    'TOO_LONG': 6,
    'SENDQ': 7,
}

TOX_ERR_GET_PORT = {
    'OK': 0,
    'NOT_BOUND': 1,
}

TOX_PUBLIC_KEY_SIZE = 32

TOX_ADDRESS_SIZE = TOX_PUBLIC_KEY_SIZE + 6

TOX_MAX_FRIEND_REQUEST_LENGTH = 1016

TOX_MAX_MESSAGE_LENGTH = 1372

TOX_MAX_NAME_LENGTH = 128

TOX_MAX_STATUS_MESSAGE_LENGTH = 1007

TOX_SECRET_KEY_SIZE = 32

TOX_FILE_ID_LENGTH = 32

TOX_HASH_LENGTH = 32

TOX_MAX_CUSTOM_PACKET_SIZE = 1373



try:
    from PySide import QtCore
except ImportError:
    from PyQt4 import QtCore
from notifications import *
from settings import Settings
from profile import Profile
from toxcore_enums_and_consts import *
from toxav_enums import *
from tox import bin_to_string
from plugin_support import PluginLoader


class InvokeEvent(QtCore.QEvent):
    EVENT_TYPE = QtCore.QEvent.Type(QtCore.QEvent.registerEventType())

    def __init__(self, fn, *args, **kwargs):
        QtCore.QEvent.__init__(self, InvokeEvent.EVENT_TYPE)
        self.fn = fn
        self.args = args
        self.kwargs = kwargs


class Invoker(QtCore.QObject):

    def event(self, event):
        event.fn(*event.args, **event.kwargs)
        return True

_invoker = Invoker()


def invoke_in_main_thread(fn, *args, **kwargs):
    QtCore.QCoreApplication.postEvent(_invoker, InvokeEvent(fn, *args, **kwargs))

# -----------------------------------------------------------------------------------------------------------------
# Callbacks - current user
# -----------------------------------------------------------------------------------------------------------------


def self_connection_status(tox_link):
    """
    Current user changed connection status (offline, UDP, TCP)
    """
    def wrapped(tox, connection, user_data):
        print('Connection status: ', str(connection))
        profile = Profile.get_instance()
        if profile.status is None:
            status = tox_link.self_get_status()
            invoke_in_main_thread(profile.set_status, status)
        elif connection == TOX_CONNECTION['NONE']:
            invoke_in_main_thread(profile.set_status, None)
    return wrapped


# -----------------------------------------------------------------------------------------------------------------
# Callbacks - friends
# -----------------------------------------------------------------------------------------------------------------


def friend_status(tox, friend_num, new_status, user_data):
    """
    Check friend's status (none, busy, away)
    """
    print("Friend's #{} status changed!".format(friend_num))
    profile = Profile.get_instance()
    friend = profile.get_friend_by_number(friend_num)
    if friend.status is None and Settings.get_instance()['sound_notifications'] and profile.status != TOX_USER_STATUS['BUSY']:
        sound_notification(SOUND_NOTIFICATION['FRIEND_CONNECTION_STATUS'])
    invoke_in_main_thread(friend.set_status, new_status)
    invoke_in_main_thread(profile.send_files, friend_num)
    invoke_in_main_thread(profile.update_filtration)


def friend_connection_status(tox, friend_num, new_status, user_data):
    """
    Check friend's connection status (offline, udp, tcp)
    """
    print("Friend #{} connection status: {}".format(friend_num, new_status))
    profile = Profile.get_instance()
    friend = profile.get_friend_by_number(friend_num)
    if new_status == TOX_CONNECTION['NONE']:
        invoke_in_main_thread(profile.friend_exit, friend_num)
        invoke_in_main_thread(profile.update_filtration)
        if Settings.get_instance()['sound_notifications'] and profile.status != TOX_USER_STATUS['BUSY']:
            sound_notification(SOUND_NOTIFICATION['FRIEND_CONNECTION_STATUS'])
    elif friend.status is None:
        invoke_in_main_thread(profile.send_avatar, friend_num)
        invoke_in_main_thread(PluginLoader.get_instance().friend_online, friend_num)


def friend_name(tox, friend_num, name, size, user_data):
    """
    Friend changed his name
    """
    profile = Profile.get_instance()
    print('New name friend #' + str(friend_num))
    invoke_in_main_thread(profile.new_name, friend_num, name)


def friend_status_message(tox, friend_num, status_message, size, user_data):
    """
    :return: function for callback friend_status_message. It updates friend's status message
    and calls window repaint
    """
    profile = Profile.get_instance()
    friend = profile.get_friend_by_number(friend_num)
    invoke_in_main_thread(friend.set_status_message, status_message)
    print('User #{} has new status'.format(friend_num))
    invoke_in_main_thread(profile.send_messages, friend_num)
    if profile.get_active_number() == friend_num:
        invoke_in_main_thread(profile.set_active)


def friend_message(window, tray):
    """
    New message from friend
    """
    def wrapped(tox, friend_number, message_type, message, size, user_data):
        profile = Profile.get_instance()
        settings = Settings.get_instance()
        message = str(message, 'utf-8')
        invoke_in_main_thread(profile.new_message, friend_number, message_type, message)
        if not window.isActiveWindow():
            friend = profile.get_friend_by_number(friend_number)
            if settings['notifications'] and profile.status != TOX_USER_STATUS['BUSY'] and not settings.locked:
                invoke_in_main_thread(tray_notification, friend.name, message, tray, window)
            if settings['sound_notifications'] and profile.status != TOX_USER_STATUS['BUSY']:
                sound_notification(SOUND_NOTIFICATION['MESSAGE'])
            invoke_in_main_thread(tray.setIcon, QtGui.QIcon(curr_directory() + '/images/icon_new_messages.png'))
    return wrapped


def friend_request(tox, public_key, message, message_size, user_data):
    """
    Called when user get new friend request
    """
    print('Friend request')
    profile = Profile.get_instance()
    key = ''.join(chr(x) for x in public_key[:TOX_PUBLIC_KEY_SIZE])
    tox_id = bin_to_string(key, TOX_PUBLIC_KEY_SIZE)
    if tox_id not in Settings.get_instance()['blocked']:
        invoke_in_main_thread(profile.process_friend_request, tox_id, str(message, 'utf-8'))


def friend_typing(tox, friend_number, typing, user_data):
    invoke_in_main_thread(Profile.get_instance().friend_typing, friend_number, typing)


def friend_read_receipt(tox, friend_number, message_id, user_data):
    profile = Profile.get_instance()
    profile.get_friend_by_number(friend_number).dec_receipt()
    if friend_number == profile.get_active_number():
        invoke_in_main_thread(profile.receipt)

# -----------------------------------------------------------------------------------------------------------------
# Callbacks - file transfers
# -----------------------------------------------------------------------------------------------------------------


def tox_file_recv(window, tray):
    """
    New incoming file
    """
    def wrapped(tox, friend_number, file_number, file_type, size, file_name, file_name_size, user_data):
        profile = Profile.get_instance()
        settings = Settings.get_instance()
        if file_type == TOX_FILE_KIND['DATA']:
            print('File')
            try:
                file_name = str(file_name[:file_name_size], 'utf-8')
            except:
                file_name = 'toxygen_file'
            invoke_in_main_thread(profile.incoming_file_transfer,
                                  friend_number,
                                  file_number,
                                  size,
                                  file_name)
            if not window.isActiveWindow():
                friend = profile.get_friend_by_number(friend_number)
                if settings['notifications'] and profile.status != TOX_USER_STATUS['BUSY'] and not settings.locked:
                    file_from = QtGui.QApplication.translate("Callback", "File from", None, QtGui.QApplication.UnicodeUTF8)
                    invoke_in_main_thread(tray_notification, file_from + ' ' + friend.name, file_name, tray, window)
                if settings['sound_notifications'] and profile.status != TOX_USER_STATUS['BUSY']:
                    sound_notification(SOUND_NOTIFICATION['FILE_TRANSFER'])
                invoke_in_main_thread(tray.setIcon, QtGui.QIcon(curr_directory() + '/images/icon_new_messages.png'))
        else:  # AVATAR
            print('Avatar')
            invoke_in_main_thread(profile.incoming_avatar,
                                  friend_number,
                                  file_number,
                                  size)
    return wrapped


def file_recv_chunk(tox, friend_number, file_number, position, chunk, length, user_data):
    """
    Incoming chunk
    """
    if not length:
        invoke_in_main_thread(Profile.get_instance().incoming_chunk,
                              friend_number,
                              file_number,
                              position,
                              None)
    else:
        Profile.get_instance().incoming_chunk(friend_number, file_number, position, chunk[:length])


def file_chunk_request(tox, friend_number, file_number, position, size, user_data):
    """
    Outgoing chunk
    """
    if size:
        Profile.get_instance().outgoing_chunk(friend_number, file_number, position, size)
    else:
        invoke_in_main_thread(Profile.get_instance().outgoing_chunk,
                              friend_number,
                              file_number,
                              position,
                              size)


def file_recv_control(tox, friend_number, file_number, file_control, user_data):
    """
    Friend cancelled, paused or resumed file transfer
    """
    if file_control == TOX_FILE_CONTROL['CANCEL']:
        invoke_in_main_thread(Profile.get_instance().cancel_transfer, friend_number, file_number, True)
    elif file_control == TOX_FILE_CONTROL['PAUSE']:
        invoke_in_main_thread(Profile.get_instance().pause_transfer, friend_number, file_number, True)
    elif file_control == TOX_FILE_CONTROL['RESUME']:
        invoke_in_main_thread(Profile.get_instance().resume_transfer, friend_number, file_number, True)

# -----------------------------------------------------------------------------------------------------------------
# Callbacks - custom packets
# -----------------------------------------------------------------------------------------------------------------


def lossless_packet(tox, friend_number, data, length, user_data):
    """
    Incoming lossless packet
    """
    plugin = PluginLoader.get_instance()
    invoke_in_main_thread(plugin.callback_lossless, friend_number, data, length)


def lossy_packet(tox, friend_number, data, length, user_data):
    """
    Incoming lossy packet
    """
    plugin = PluginLoader.get_instance()
    invoke_in_main_thread(plugin.callback_lossy, friend_number, data, length)


# -----------------------------------------------------------------------------------------------------------------
# Callbacks - audio
# -----------------------------------------------------------------------------------------------------------------

def call_state(toxav, friend_number, mask, user_data):
    """
    New call state
    """
    print(friend_number, mask)
    if mask == TOXAV_FRIEND_CALL_STATE['FINISHED'] or mask == TOXAV_FRIEND_CALL_STATE['ERROR']:
        invoke_in_main_thread(Profile.get_instance().stop_call, friend_number, True)
    else:
        Profile.get_instance().call.toxav_call_state_cb(friend_number, mask)


def call(toxav, friend_number, audio, video, user_data):
    """
    Incoming call from friend
    """
    print(friend_number, audio, video)
    invoke_in_main_thread(Profile.get_instance().incoming_call, audio, video, friend_number)


def callback_audio(toxav, friend_number, samples, audio_samples_per_channel, audio_channels_count, rate, user_data):
    """
    New audio chunk
    """
    # print(audio_samples_per_channel, audio_channels_count, rate)
    Profile.get_instance().call.chunk(
        bytes(samples[:audio_samples_per_channel * 2 * audio_channels_count]),
        audio_channels_count,
        rate)


# -----------------------------------------------------------------------------------------------------------------
# Callbacks - initialization
# -----------------------------------------------------------------------------------------------------------------


def init_callbacks(tox, window, tray):
    """
    Initialization of all callbacks.
    :param tox: tox instance
    :param window: main window
    :param tray: tray (for notifications)
    """
    tox.callback_self_connection_status(self_connection_status(tox), 0)

    tox.callback_friend_status(friend_status, 0)
    tox.callback_friend_message(friend_message(window, tray), 0)
    tox.callback_friend_connection_status(friend_connection_status, 0)
    tox.callback_friend_name(friend_name, 0)
    tox.callback_friend_status_message(friend_status_message, 0)
    tox.callback_friend_request(friend_request, 0)
    tox.callback_friend_typing(friend_typing, 0)
    tox.callback_friend_read_receipt(friend_read_receipt, 0)

    tox.callback_file_recv(tox_file_recv(window, tray), 0)
    tox.callback_file_recv_chunk(file_recv_chunk, 0)
    tox.callback_file_chunk_request(file_chunk_request, 0)
    tox.callback_file_recv_control(file_recv_control, 0)

    toxav = tox.AV
    toxav.callback_call_state(call_state, 0)
    toxav.callback_call(call, 0)
    toxav.callback_audio_receive_frame(callback_audio, 0)

    tox.callback_friend_lossless_packet(lossless_packet, 0)
    tox.callback_friend_lossy_packet(lossy_packet, 0)




try:
    from PySide import QtCore, QtGui
except ImportError:
    from PyQt4 import QtCore, QtGui
from util import curr_directory
import wave
import pyaudio


SOUND_NOTIFICATION = {
    'MESSAGE': 0,
    'FRIEND_CONNECTION_STATUS': 1,
    'FILE_TRANSFER': 2
}


def tray_notification(title, text, tray, window):
    """
    Show tray notification and activate window icon
    NOTE: different behaviour on different OS
    :param title: Name of user who sent message or file
    :param text: text of message or file info
    :param tray: ref to tray icon
    :param window: main window
    """
    if QtGui.QSystemTrayIcon.isSystemTrayAvailable():
        if len(text) > 30:
            text = text[:27] + '...'
        tray.showMessage(title, text, QtGui.QSystemTrayIcon.NoIcon, 3000)
        QtGui.QApplication.alert(window, 0)

        def message_clicked():
            window.setWindowState(window.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
            window.activateWindow()
        tray.connect(tray, QtCore.SIGNAL("messageClicked()"), message_clicked)


class AudioFile:
    chunk = 1024

    def __init__(self, fl):
        self.wf = wave.open(fl, 'rb')
        self.p = pyaudio.PyAudio()
        self.stream = self.p.open(
            format=self.p.get_format_from_width(self.wf.getsampwidth()),
            channels=self.wf.getnchannels(),
            rate=self.wf.getframerate(),
            output=True
        )

    def play(self):
        data = self.wf.readframes(self.chunk)
        while data:
            self.stream.write(data)
            data = self.wf.readframes(self.chunk)

    def close(self):
        self.stream.close()
        self.p.terminate()


def sound_notification(t):
    """
    Plays sound notification
    :param t: type of notification
    """
    if t == SOUND_NOTIFICATION['MESSAGE']:
        f = curr_directory() + '/sounds/message.wav'
    elif t == SOUND_NOTIFICATION['FILE_TRANSFER']:
        f = curr_directory() + '/sounds/file.wav'
    else:
        f = curr_directory() + '/sounds/contact.wav'
    a = AudioFile(f)
    a.play()
    a.close()



import sys
from loginscreen import LoginScreen
import profile
from settings import *
try:
    from PySide import QtCore, QtGui
except ImportError:
    from PyQt4 import QtCore, QtGui
from bootstrap import node_generator
from mainscreen import MainWindow
from callbacks import init_callbacks
from util import curr_directory, program_version
import styles.style
import platform
import toxencryptsave
from passwordscreen import PasswordScreen, UnlockAppScreen, SetProfilePasswordScreen
from plugin_support import PluginLoader


class Toxygen:

    def __init__(self, path_or_uri=None):
        super(Toxygen, self).__init__()
        self.tox = self.ms = self.init = self.mainloop = self.avloop = None
        if path_or_uri is None:
            self.uri = self.path = None
        elif path_or_uri.startswith('tox:'):
            self.path = None
            self.uri = path_or_uri[4:]
        else:
            self.path = path_or_uri
            self.uri = None

    def enter_pass(self, data):
        """
        Show password screen
        """
        tmp = [data]
        p = PasswordScreen(toxencryptsave.ToxEncryptSave.get_instance(), tmp)
        p.show()
        self.app.connect(self.app, QtCore.SIGNAL("lastWindowClosed()"), self.app, QtCore.SLOT("quit()"))
        self.app.exec_()
        if tmp[0] == data:
            raise SystemExit()
        else:
            return tmp[0]

    def main(self):
        """
        Main function of app. loads login screen if needed and starts main screen
        """
        app = QtGui.QApplication(sys.argv)
        app.setWindowIcon(QtGui.QIcon(curr_directory() + '/images/icon.png'))
        self.app = app

        if platform.system() == 'Linux':
            QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_X11InitThreads)

        # application color scheme
        with open(curr_directory() + '/styles/style.qss') as fl:
            dark_style = fl.read()
        app.setStyleSheet(dark_style)

        encrypt_save = toxencryptsave.ToxEncryptSave()

        if self.path is not None:
            path = os.path.dirname(self.path) + '/'
            name = os.path.basename(self.path)[:-4]
            data = ProfileHelper(path, name).open_profile()
            if encrypt_save.is_data_encrypted(data):
                data = self.enter_pass(data)
            settings = Settings(name)
            self.tox = profile.tox_factory(data, settings)
        else:
            auto_profile = Settings.get_auto_profile()
            if not auto_profile[0]:
                # show login screen if default profile not found
                current_locale = QtCore.QLocale()
                curr_lang = current_locale.languageToString(current_locale.language())
                langs = Settings.supported_languages()
                if curr_lang in langs:
                    lang_path = langs[curr_lang]
                    translator = QtCore.QTranslator()
                    translator.load(curr_directory() + '/translations/' + lang_path)
                    app.installTranslator(translator)
                    app.translator = translator
                ls = LoginScreen()
                ls.setWindowIconText("Toxygen")
                profiles = ProfileHelper.find_profiles()
                ls.update_select(map(lambda x: x[1], profiles))
                _login = self.Login(profiles)
                ls.update_on_close(_login.login_screen_close)
                ls.show()
                app.connect(app, QtCore.SIGNAL("lastWindowClosed()"), app, QtCore.SLOT("quit()"))
                app.exec_()
                if not _login.t:
                    return
                elif _login.t == 1:  # create new profile
                    _login.name = _login.name.strip()
                    name = _login.name if _login.name else 'toxygen_user'
                    pr = map(lambda x: x[1], ProfileHelper.find_profiles())
                    if name in list(pr):
                        msgBox = QtGui.QMessageBox()
                        msgBox.setWindowTitle(
                            QtGui.QApplication.translate("MainWindow", "Error", None, QtGui.QApplication.UnicodeUTF8))
                        text = (QtGui.QApplication.translate("MainWindow",
                                                             'Profile with this name already exists',
                                                             None, QtGui.QApplication.UnicodeUTF8))
                        msgBox.setText(text)
                        msgBox.exec_()
                        return
                    self.tox = profile.tox_factory()
                    self.tox.self_set_name(bytes(_login.name, 'utf-8') if _login.name else b'Toxygen User')
                    self.tox.self_set_status_message(b'Toxing on Toxygen')
                    reply = QtGui.QMessageBox.question(None,
                                                       'Profile {}'.format(name),
                                                       QtGui.QApplication.translate("login",
                                                                                    'Do you want to set profile password?',
                                                                                    None,
                                                                                    QtGui.QApplication.UnicodeUTF8),
                                                       QtGui.QMessageBox.Yes,
                                                       QtGui.QMessageBox.No)
                    if reply == QtGui.QMessageBox.Yes:
                        set_pass = SetProfilePasswordScreen(encrypt_save)
                        set_pass.show()
                        self.app.connect(self.app, QtCore.SIGNAL("lastWindowClosed()"), self.app, QtCore.SLOT("quit()"))
                        self.app.exec_()
                    ProfileHelper(Settings.get_default_path(), name).save_profile(self.tox.get_savedata())
                    path = Settings.get_default_path()
                    settings = Settings(name)
                    if curr_lang in langs:
                        settings['language'] = curr_lang
                    settings.save()
                else:  # load existing profile
                    path, name = _login.get_data()
                    if _login.default:
                        Settings.set_auto_profile(path, name)
                    data = ProfileHelper(path, name).open_profile()
                    if encrypt_save.is_data_encrypted(data):
                        data = self.enter_pass(data)
                    settings = Settings(name)
                    self.tox = profile.tox_factory(data, settings)
            else:
                path, name = auto_profile
                data = ProfileHelper(path, name).open_profile()
                if encrypt_save.is_data_encrypted(data):
                    data = self.enter_pass(data)
                settings = Settings(name)
                self.tox = profile.tox_factory(data, settings)

        if Settings.is_active_profile(path, name):  # profile is in use
            reply = QtGui.QMessageBox.question(None,
                                               'Profile {}'.format(name),
                                               QtGui.QApplication.translate("login", 'Other instance of Toxygen uses this profile or profile was not properly closed. Continue?', None, QtGui.QApplication.UnicodeUTF8),
                                               QtGui.QMessageBox.Yes,
                                               QtGui.QMessageBox.No)
            if reply != QtGui.QMessageBox.Yes:
                return
        else:
            settings.set_active_profile()

        lang = Settings.supported_languages()[settings['language']]
        translator = QtCore.QTranslator()
        translator.load(curr_directory() + '/translations/' + lang)
        app.installTranslator(translator)
        app.translator = translator

        # tray icon
        self.tray = QtGui.QSystemTrayIcon(QtGui.QIcon(curr_directory() + '/images/icon.png'))
        self.tray.setObjectName('tray')

        self.ms = MainWindow(self.tox, self.reset, self.tray)

        class Menu(QtGui.QMenu):

            def newStatus(self, status):
                profile.Profile.get_instance().set_status(status)
                self.aboutToShow()
                self.hide()

            def aboutToShow(self):
                status = profile.Profile.get_instance().status
                act = self.act
                if status is None or Settings.get_instance().locked:
                    self.actions()[1].setVisible(False)
                else:
                    self.actions()[1].setVisible(True)
                    act.actions()[0].setChecked(False)
                    act.actions()[1].setChecked(False)
                    act.actions()[2].setChecked(False)
                    act.actions()[status].setChecked(True)
                self.actions()[2].setVisible(not Settings.get_instance().locked)

            def languageChange(self, *args, **kwargs):
                self.actions()[0].setText(QtGui.QApplication.translate('tray', 'Open Toxygen', None, QtGui.QApplication.UnicodeUTF8))
                self.actions()[1].setText(QtGui.QApplication.translate('tray', 'Set status', None, QtGui.QApplication.UnicodeUTF8))
                self.actions()[2].setText(QtGui.QApplication.translate('tray', 'Exit', None, QtGui.QApplication.UnicodeUTF8))
                self.act.actions()[0].setText(QtGui.QApplication.translate('tray', 'Online', None, QtGui.QApplication.UnicodeUTF8))
                self.act.actions()[1].setText(QtGui.QApplication.translate('tray', 'Away', None, QtGui.QApplication.UnicodeUTF8))
                self.act.actions()[2].setText(QtGui.QApplication.translate('tray', 'Busy', None, QtGui.QApplication.UnicodeUTF8))

        m = Menu()
        show = m.addAction(QtGui.QApplication.translate('tray', 'Open Toxygen', None, QtGui.QApplication.UnicodeUTF8))
        sub = m.addMenu(QtGui.QApplication.translate('tray', 'Set status', None, QtGui.QApplication.UnicodeUTF8))
        onl = sub.addAction(QtGui.QApplication.translate('tray', 'Online', None, QtGui.QApplication.UnicodeUTF8))
        away = sub.addAction(QtGui.QApplication.translate('tray', 'Away', None, QtGui.QApplication.UnicodeUTF8))
        busy = sub.addAction(QtGui.QApplication.translate('tray', 'Busy', None, QtGui.QApplication.UnicodeUTF8))
        onl.setCheckable(True)
        away.setCheckable(True)
        busy.setCheckable(True)
        m.act = sub
        exit = m.addAction(QtGui.QApplication.translate('tray', 'Exit', None, QtGui.QApplication.UnicodeUTF8))

        def show_window():
            def show():
                if not self.ms.isActiveWindow():
                    self.ms.setWindowState(self.ms.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
                    self.ms.activateWindow()
                self.ms.show()
            if not Settings.get_instance().locked:
                show()
            else:
                def correct_pass():
                    show()
                    Settings.get_instance().locked = False
                self.p = UnlockAppScreen(toxencryptsave.ToxEncryptSave.get_instance(), correct_pass)
                self.p.show()

        m.connect(show, QtCore.SIGNAL("triggered()"), show_window)
        m.connect(exit, QtCore.SIGNAL("triggered()"), lambda: app.exit())
        m.connect(m, QtCore.SIGNAL("aboutToShow()"), lambda: m.aboutToShow())
        sub.connect(onl, QtCore.SIGNAL("triggered()"), lambda: m.newStatus(0))
        sub.connect(away, QtCore.SIGNAL("triggered()"), lambda: m.newStatus(1))
        sub.connect(busy, QtCore.SIGNAL("triggered()"), lambda: m.newStatus(2))

        self.tray.setContextMenu(m)
        self.tray.show()

        self.ms.show()

        plugin_helper = PluginLoader(self.tox, settings)  # plugin support
        plugin_helper.load()

        # init thread
        self.init = self.InitThread(self.tox, self.ms, self.tray)
        self.init.start()

        # starting threads for tox iterate and toxav iterate
        self.mainloop = self.ToxIterateThread(self.tox)
        self.mainloop.start()
        self.avloop = self.ToxAVIterateThread(self.tox.AV)
        self.avloop.start()

        if self.uri is not None:
            self.ms.add_contact(self.uri)

        app.connect(app, QtCore.SIGNAL("lastWindowClosed()"), app, QtCore.SLOT("quit()"))
        app.exec_()
        self.init.stop = True
        self.mainloop.stop = True
        self.avloop.stop = True
        plugin_helper.stop()
        self.mainloop.wait()
        self.init.wait()
        self.avloop.wait()
        data = self.tox.get_savedata()
        ProfileHelper.get_instance().save_profile(data)
        settings.close()
        del self.tox

    def reset(self):
        """
        Create new tox instance (new network settings)
        :return: tox instance
        """
        self.mainloop.stop = True
        self.init.stop = True
        self.avloop.stop = True
        self.mainloop.wait()
        self.init.wait()
        self.avloop.wait()
        data = self.tox.get_savedata()
        ProfileHelper.get_instance().save_profile(data)
        del self.tox
        # create new tox instance
        self.tox = profile.tox_factory(data, Settings.get_instance())
        # init thread
        self.init = self.InitThread(self.tox, self.ms, self.tray)
        self.init.start()

        # starting threads for tox iterate and toxav iterate
        self.mainloop = self.ToxIterateThread(self.tox)
        self.mainloop.start()

        self.avloop = self.ToxAVIterateThread(self.tox.AV)
        self.avloop.start()

        plugin_helper = PluginLoader.get_instance()
        plugin_helper.set_tox(self.tox)

        return self.tox

    # -----------------------------------------------------------------------------------------------------------------
    # Inner classes
    # -----------------------------------------------------------------------------------------------------------------

    class InitThread(QtCore.QThread):

        def __init__(self, tox, ms, tray):
            QtCore.QThread.__init__(self)
            self.tox, self.ms, self.tray = tox, ms, tray
            self.stop = False

        def run(self):
            # initializing callbacks
            init_callbacks(self.tox, self.ms, self.tray)
            # bootstrap
            try:
                for data in node_generator():
                    if self.stop:
                        return
                    self.tox.bootstrap(*data)
                    self.tox.add_tcp_relay(*data)
            except:
                pass
            for _ in range(10):
                if self.stop:
                    return
                self.msleep(1000)
            while not self.tox.self_get_connection_status():
                try:
                    for data in node_generator():
                        if self.stop:
                            return
                        self.tox.bootstrap(*data)
                        self.tox.add_tcp_relay(*data)
                except:
                    pass
                finally:
                    self.msleep(5000)

    class ToxIterateThread(QtCore.QThread):

        def __init__(self, tox):
            QtCore.QThread.__init__(self)
            self.tox = tox
            self.stop = False

        def run(self):
            while not self.stop:
                self.tox.iterate()
                self.msleep(self.tox.iteration_interval())

    class ToxAVIterateThread(QtCore.QThread):

        def __init__(self, toxav):
            QtCore.QThread.__init__(self)
            self.toxav = toxav
            self.stop = False

        def run(self):
            while not self.stop:
                self.toxav.iterate()
                self.msleep(self.toxav.iteration_interval())

    class Login:

        def __init__(self, arr):
            self.arr = arr

        def login_screen_close(self, t, number=-1, default=False, name=None):
            """ Function which processes data from login screen
            :param t: 0 - window was closed, 1 - new profile was created, 2 - profile loaded
            :param number: num of chosen profile in list (-1 by default)
            :param default: was or not chosen profile marked as default
            :param name: name of new profile
            """
            self.t = t
            self.num = number
            self.default = default
            self.name = name

        def get_data(self):
            return self.arr[self.num]


def clean():
    """Removes all windows libs from libs folder"""
    d = curr_directory() + '/libs/'
    for fl in ('libtox64.dll', 'libtox.dll', 'libsodium64.a', 'libsodium.a'):
        if os.path.exists(d + fl):
            os.remove(d + fl)


def configure():
    """Removes unused libs"""
    d = curr_directory() + '/libs/'
    is_64bits = sys.maxsize > 2 ** 32
    if not is_64bits:
        if os.path.exists(d + 'libtox64.dll'):
            os.remove(d + 'libtox64.dll')
        if os.path.exists(d + 'libsodium64.a'):
            os.remove(d + 'libsodium64.a')
    else:
        if os.path.exists(d + 'libtox.dll'):
            os.remove(d + 'libtox.dll')
        if os.path.exists(d + 'libsodium.a'):
            os.remove(d + 'libsodium.a')
        try:
            os.rename(d + 'libtox64.dll', d + 'libtox.dll')
            os.rename(d + 'libsodium64.a', d + 'libsodium.a')
        except:
            pass


def main():
    if len(sys.argv) == 1:
        toxygen = Toxygen()
    else:  # started with argument(s)
        arg = sys.argv[1]
        if arg == '--version':
            print('Toxygen ' + program_version)
            return
        elif arg == '--help':
            print('Usage:\ntoxygen path_to_profile\ntoxygen tox_id\ntoxygen --version')
            return
        elif arg == '--configure':
            configure()
            return
        elif arg == '--clean':
            clean()
            return
        else:
            toxygen = Toxygen(arg)
    toxygen.main()


if __name__ == '__main__':
    main()



from toxcore_enums_and_consts import TOX_FILE_KIND, TOX_FILE_CONTROL
from os.path import basename, getsize, exists, dirname
from os import remove, rename, chdir
from time import time, sleep
from tox import Tox
import settings
try:
    from PySide import QtCore
except ImportError:
    from PyQt4 import QtCore

# TODO: threads!


TOX_FILE_TRANSFER_STATE = {
    'RUNNING': 0,
    'PAUSED_BY_USER': 1,
    'CANCELLED': 2,
    'FINISHED': 3,
    'PAUSED_BY_FRIEND': 4,
    'INCOMING_NOT_STARTED': 5,
    'OUTGOING_NOT_STARTED': 6
}

ACTIVE_FILE_TRANSFERS = (0, 1, 4, 5, 6)

PAUSED_FILE_TRANSFERS = (1, 4, 5, 6)

DO_NOT_SHOW_ACCEPT_BUTTON = (2, 3, 4, 6)

SHOW_PROGRESS_BAR = (0, 1, 4)

ALLOWED_FILES = ('toxygen_inline.png', 'utox-inline.png', 'sticker.png')


class StateSignal(QtCore.QObject):
    try:
        signal = QtCore.Signal(int, float, int)  # state and progress
    except:
        signal = QtCore.pyqtSignal(int, float, int)  # state and progress - pyqt4


class FileTransfer(QtCore.QObject):
    """
    Superclass for file transfers
    """

    def __init__(self, path, tox, friend_number, size, file_number=None):
        QtCore.QObject.__init__(self)
        self._path = path
        self._tox = tox
        self._friend_number = friend_number
        self.state = TOX_FILE_TRANSFER_STATE['RUNNING']
        self._file_number = file_number
        self._creation_time = None
        self._size = float(size)
        self._done = 0
        self._state_changed = StateSignal()

    def set_tox(self, tox):
        self._tox = tox

    def set_state_changed_handler(self, handler):
        self._state_changed.signal.connect(handler)

    def signal(self):
        percentage = self._done / self._size if self._size else 0
        if self._creation_time is None or not percentage:
            t = -1
        else:
            t = ((time() - self._creation_time) / percentage) * (1 - percentage)
        self._state_changed.signal.emit(self.state, percentage, int(t))

    def get_file_number(self):
        return self._file_number

    def get_friend_number(self):
        return self._friend_number

    def cancel(self):
        self.send_control(TOX_FILE_CONTROL['CANCEL'])
        if hasattr(self, '_file'):
            self._file.close()
        self.signal()

    def cancelled(self):
        if hasattr(self, '_file'):
            sleep(0.1)
            self._file.close()
        self.state = TOX_FILE_TRANSFER_STATE['CANCELLED']
        self.signal()

    def pause(self, by_friend):
        if not by_friend:
            self.send_control(TOX_FILE_CONTROL['PAUSE'])
        else:
            self.state = TOX_FILE_TRANSFER_STATE['PAUSED_BY_FRIEND']
        self.signal()

    def send_control(self, control):
        if self._tox.file_control(self._friend_number, self._file_number, control):
            self.state = control
            self.signal()

    def get_file_id(self):
        return self._tox.file_get_file_id(self._friend_number, self._file_number)

# -----------------------------------------------------------------------------------------------------------------
# Send file
# -----------------------------------------------------------------------------------------------------------------


class SendTransfer(FileTransfer):

    def __init__(self, path, tox, friend_number, kind=TOX_FILE_KIND['DATA'], file_id=None):
        if path is not None:
            self._file = open(path, 'rb')
            size = getsize(path)
        else:
            size = 0
        super(SendTransfer, self).__init__(path, tox, friend_number, size)
        self.state = TOX_FILE_TRANSFER_STATE['OUTGOING_NOT_STARTED']
        self._file_number = tox.file_send(friend_number, kind, size, file_id,
                                          bytes(basename(path), 'utf-8') if path else b'')

    def send_chunk(self, position, size):
        """
        Send chunk
        :param position: start position in file
        :param size: chunk max size
        """
        if self._creation_time is None:
            self._creation_time = time()
        if size:
            self._file.seek(position)
            data = self._file.read(size)
            self._tox.file_send_chunk(self._friend_number, self._file_number, position, data)
            self._done += size
            self.signal()
        else:
            if hasattr(self, '_file'):
                self._file.close()
            self.state = TOX_FILE_TRANSFER_STATE['FINISHED']
            self.signal()


class SendAvatar(SendTransfer):
    """
    Send avatar to friend. Doesn't need file transfer item
    """

    def __init__(self, path, tox, friend_number):
        if path is None:
            hash = None
        else:
            with open(path, 'rb') as fl:
                hash = Tox.hash(fl.read())
        super(SendAvatar, self).__init__(path, tox, friend_number, TOX_FILE_KIND['AVATAR'], hash)


class SendFromBuffer(FileTransfer):
    """
    Send inline image
    """

    def __init__(self, tox, friend_number, data, file_name):
        super(SendFromBuffer, self).__init__(None, tox, friend_number, len(data))
        self.state = TOX_FILE_TRANSFER_STATE['OUTGOING_NOT_STARTED']
        self._data = data
        self._file_number = tox.file_send(friend_number, TOX_FILE_KIND['DATA'],
                                          len(data), None, bytes(file_name, 'utf-8'))

    def get_data(self):
        return self._data

    def send_chunk(self, position, size):
        if self._creation_time is None:
            self._creation_time = time()
        if size:
            data = self._data[position:position + size]
            self._tox.file_send_chunk(self._friend_number, self._file_number, position, data)
            self._done += size
            self.signal()
        else:
            self.state = TOX_FILE_TRANSFER_STATE['FINISHED']
            self.signal()


class SendFromFileBuffer(SendTransfer):

    def __init__(self, *args):
        super(SendFromFileBuffer, self).__init__(*args)

    def send_chunk(self, position, size):
        super(SendFromFileBuffer, self).send_chunk(position, size)
        if not size:
            chdir(dirname(self._path))
            remove(self._path)

# -----------------------------------------------------------------------------------------------------------------
# Receive file
# -----------------------------------------------------------------------------------------------------------------


class ReceiveTransfer(FileTransfer):

    def __init__(self, path, tox, friend_number, size, file_number):
        super(ReceiveTransfer, self).__init__(path, tox, friend_number, size, file_number)
        self._file = open(self._path, 'wb')
        self._file.truncate(0)
        self._file_size = 0

    def cancel(self):
        super(ReceiveTransfer, self).cancel()
        remove(self._path)

    def write_chunk(self, position, data):
        """
        Incoming chunk
        :param position: position in file to save data
        :param data: raw data (string)
        """
        if self._creation_time is None:
            self._creation_time = time()
        if data is None:
            self._file.close()
            self.state = TOX_FILE_TRANSFER_STATE['FINISHED']
            self.signal()
        else:
            data = bytearray(data)
            if self._file_size < position:
                self._file.seek(0, 2)
                self._file.write(b'\0' * (position - self._file_size))
            self._file.seek(position)
            self._file.write(data)
            l = len(data)
            if position + l > self._file_size:
                self._file_size = position + l
            self._done += l
            self.signal()


class ReceiveToBuffer(FileTransfer):
    """
    Inline image - save in buffer not in file system
    """

    def __init__(self, tox, friend_number, size, file_number):
        super(ReceiveToBuffer, self).__init__(None, tox, friend_number, size, file_number)
        self._data = bytes()
        self._data_size = 0

    def get_data(self):
        return self._data

    def write_chunk(self, position, data):
        if self._creation_time is None:
            self._creation_time = time()
        if data is None:
            self.state = TOX_FILE_TRANSFER_STATE['FINISHED']
        else:
            data = bytes(data)
            l = len(data)
            if self._data_size < position:
                self._data += (b'\0' * (position - self._data_size))
            self._data = self._data[:position] + data + self._data[position + l:]
            if position + l > self._data_size:
                self._data_size = position + l
            self._done += l
        self.signal()


class ReceiveAvatar(ReceiveTransfer):
    """
    Get friend's avatar. Doesn't need file transfer item
    """
    MAX_AVATAR_SIZE = 512 * 1024

    def __init__(self, tox, friend_number, size, file_number):
        path = settings.ProfileHelper.get_path() + 'avatars/{}.png'.format(tox.friend_get_public_key(friend_number))
        super(ReceiveAvatar, self).__init__(path + '.tmp', tox, friend_number, size, file_number)
        if size > self.MAX_AVATAR_SIZE:
            self.send_control(TOX_FILE_CONTROL['CANCEL'])
            self._file.close()
            remove(path + '.tmp')
        elif not size:
            self.send_control(TOX_FILE_CONTROL['CANCEL'])
            self._file.close()
            if exists(path):
                remove(path)
            self._file.close()
            remove(path + '.tmp')
        elif exists(path):
            hash = self.get_file_id()
            with open(path, 'rb') as fl:
                data = fl.read()
            existing_hash = Tox.hash(data)
            if hash == existing_hash:
                self.send_control(TOX_FILE_CONTROL['CANCEL'])
                self._file.close()
                remove(path + '.tmp')
            else:
                self.send_control(TOX_FILE_CONTROL['RESUME'])
        else:
            self.send_control(TOX_FILE_CONTROL['RESUME'])

    def write_chunk(self, position, data):
        super(ReceiveAvatar, self).write_chunk(position, data)
        if self.state:
            avatar_path = self._path[:-4]
            if exists(avatar_path):
                chdir(dirname(avatar_path))
                remove(avatar_path)
            rename(self._path, avatar_path)
